setwd("~/OneDrive/Documents/Tex/2021/RPM/R MCMC") #change to your folder
library(readxl)  # allows reading excel files; assumes there is a header row
library("loo")
library(rstan)
rstan_options(auto_write = TRUE)
options(mc.cores = parallel::detectCores())
# last two  lines are some recommended Stan options
# now do one of these:
dmat = as.matrix(read_excel('dsgn_mcmc_cube.xlsx'))
dmat = as.matrix(read_excel('dsgn_mcmc_line.xlsx'))

y = scan('ln_tri.txt')
N <- nrow(dmat)
U <- ncol(dmat)
c(N,U)
df = list(N=N,U=U,dmat=dmat,y=y)

#now run stan
set.seed(8) #Only fit one chain here for examples but usually do more, like 4
fit_1 <- stan(file = 'ln_tri_spline.stan', data=df, verbose = FALSE, chains = 1, iter = 2000, control = list(adapt_delta = 0.94, max_treedepth = 20))
fit_2 <- stan(file = 'ln_tri_spline.stan', data=df, verbose = FALSE, chains = 1, iter = 6000, warmup = 4000, control = list(adapt_delta = 0.9, max_treedepth = 18))

 #compute loo
log_spread_1 <- extract_log_lik(fit_1) 
loo_spread_1 <- loo(log_spread_1)
loo_spread_1

log_spread_2 <- extract_log_lik(fit_2) 
loo_spread_2 <- loo(log_spread_2)
loo_spread_2

print(fit_1, pars=c("lncn","v","sig","s", "logs"), probs=c(.025, 0.2, 0.5, 0.8, 0.975), digits_summary = 4)
print(fit_2, pars=c("lncn","v","sig","s", "logs"), probs=c(.025, 0.2, 0.5, 0.8, 0.975), digits_summary = 4)

loo_compare(loo_spread_1,loo_spread_2)

out_fit_1 <- get_posterior_mean(fit_1)
write.csv(out_fit_1, file="out_linear_log_tri.csv")

out_fit_2 <- get_posterior_mean(fit_2)
write.csv(out_fit_2, file="out_cubic_log_tri.csv")

#_______________________________________________________
#Now fit unlogged triangle with gamma residuals. Do one:
dmat = as.matrix(read_excel('dsgn_tri_cube.xlsx'))  #all columns
dmat = as.matrix(read_excel('dsgn_mcmc_cube.xlsx')) #small params left out
dmat = as.matrix(read_excel('dsgn_mcmc_line.xlsx'))

y = scan('tri_data.txt')
initial = scan('initial.txt') #for cubic - needs starting values

N <- nrow(dmat)
U <- ncol(dmat)
c(N,U)
df = list(N=N,U=U,dmat=dmat,y=y)
#now for cubic starting values
lncn = initial[1]
v = initial[2:29]
logs = initial[30]
logbeta = initial[31]
initf = list(lncn=lncn,v=v,logs=logs,logbeta=logbeta)

set.seed(8)
fit_1 <- stan(file = 'tri_spline.stan', data=df, verbose = FALSE, chains = 1, iter = 2000, control = list(adapt_delta = 0.94, max_treedepth = 20))

set.seed(8) #putting in starting values for cubic
fit_2 <- stan(file = 'tri_spline.stan', data=df, verbose = FALSE, chains = 1, iter = 1000, warmup = 500, init = list(start1=initf), control = list(adapt_delta = 0.9, max_treedepth = 18))

print(fit_1, pars=c("lncn","v","beta","s"), probs=c(.025, 0.2, 0.5, 0.8, 0.975), digits_summary = 4)
print(fit_2, pars=c("lncn","v","beta","s"), probs=c(.025, 0.2, 0.5, 0.8, 0.975), digits_summary = 4)

log_spread_1 <- extract_log_lik(fit_1) 
loo_spread_1 <- loo(log_spread_1)
loo_spread_1

log_spread_2 <- extract_log_lik(fit_2) 
loo_spread_2 <- loo(log_spread_2)
loo_spread_2

loo_compare(loo_spread_1,loo_spread_2)

out_fit_1 <- get_posterior_mean(fit_1)
write.csv(out_fit_1, file="out_linear_gamma_tri.csv")

out_fit_2 <- get_posterior_mean(fit_2)
write.csv(out_fit_2, file="out_cubic_gamma_tri.csv")

# In case you want all the MCMC samples for simulation or analysis
fit_1_ss = extract(fit_1, permuted = FALSE) #– this gets all the samples
#Need permuted = FALSE to get it in array form
dim(fit_1_ss) # shows dimension, like: 1000 x 1 x 386
# Maybe only want 1st 24 rows, with the parameters
fit_1_ss = fit_1_ss[,,1:24]
dim(fit_1_ss)
write.csv(fit_1_ss, file="samples_spline.csv")
#with more chains often best to output one chain at a time
# write.csv(fit_1_ss[,3,], file="samples_3_spline.csv")


