data {
  int N;           //number of observations
  int U;           //number of variables
  matrix[N,U] dmat;  //design matrix with U columns
  vector[N] y;     //the triangle in a column
}
parameters {  // all except v will get uniform prior in specified range, which is default         
 real<lower=-20, upper=20> lncn;           //ln constant term, starting in known range
  vector[U] v;                          //the parameters
 real<lower=-5, upper = 5> logsig;   
 real<lower=-5, upper = 5> logs;     //log of s, related to lambda, not too high
} 
transformed parameters {
 real s;                          //shrinkage parameter, like lambda
 real sig;
 vector[N] yhat;
 sig = exp(logsig); //for positive parameter, uniform on log is like 1/X,
 s = exp(logs);     //Gives more weight to lower values, which is good if X not big
 yhat = dmat*v+exp(lncn); //fitted mean by obs 
}
model {  // gives priors for those not assumed uniform. This one for lasso.
   for (i in 1:U)  v[i] ~ double_exponential(0, s);  
for (j in 1:N) y[j] ~ normal(yhat[j], sig);  
} 
generated quantities { //outputs log likelihood for loo
  vector[N] log_lik;
for (j in 1:N) log_lik[j] = normal_lpdf(y[j] | yhat[j],sig);
}


