data {
  int N;            //number of observations
  int U;            //number of variables
  matrix[N,U] dmat; //design matrix with U columns
  vector[N] y;      //the triangle in a column
}
parameters {  // all except v will get uniform prior, which is default         
 real<lower=-20, upper=20> lncn;           //ln constant term, starting in known range
  vector[U] v;                          //the parameters
 real<lower=-5, upper = 5> logs;     //log of s, related to lambda, not too high
 real<lower=-20, upper = 20> logbeta;   //log of gamma b parameter
} 
transformed parameters {
 real beta;                       //variance in Stan = mean/beta
 real s;                          //shrinkage parameter, like lambda
 vector[N] alpha;                 //fitted alpha parameter vector 
 vector[N] yhat;
 beta = exp(logbeta); //for positive parameter, uniform on log is like 1/X,
 s = exp(logs);       //Gives more weight to lower values, which is good if X not big
 yhat = exp(dmat*v+lncn); 
 alpha = yhat*beta; //In Stan, mean = a/b
}
model {  // gives priors for those not assumed uniform. Choose this one for lasso.
   for (i in 1:U)  v[i] ~ double_exponential(0, s);  
for (j in 1:N) y[j] ~ gamma(alpha[j], beta);  //Stan gamma mean is a/b
} 
generated quantities { //outputs log likelihood for loo
  vector[N] log_lik;
for (j in 1:N) log_lik[j] = gamma_lpdf(y[j] | alpha[j],beta);
}


